﻿// Copyright 2009-2022 Josh Close
// This file is a part of CsvHelper and is dual licensed under MS-PL and Apache 2.0.
// See LICENSE.txt for details or visit http://www.opensource.org/licenses/ms-pl.html for MS-PL and http://opensource.org/licenses/Apache-2.0 for Apache 2.0.
// https://github.com/JoshClose/CsvHelper
using System;

namespace CsvHelper.Configuration.Attributes
{
	/// <summary>
	/// The mode.
	/// See <see cref="CsvMode"/> for more details.
	/// </summary>
	[AttributeUsage(AttributeTargets.Class, AllowMultiple = false, Inherited = true)]
	public class ModeAttribute : Attribute, IClassMapper
	{
		/// <summary>
		/// The mode.
		/// See <see cref="CsvMode"/> for more details.
		/// </summary>
		public CsvMode Mode { get; private set; }

		/// <summary>
		/// The mode.
		/// See <see cref="CsvMode"/> for more details.
		/// </summary>
		/// <param name="mode"></param>
		public ModeAttribute(CsvMode mode)
		{
			Mode = mode;
		}

		/// <inheritdoc />
		public void ApplyTo(CsvConfiguration configuration)
		{
			configuration.Mode = Mode;
		}
	}
}
